# Copyright (C) 2015 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from neutron.api import extensions
from neutron.api.v2 import base
from neutron.common import exceptions as nexception
from neutron import manager

from midonet.neutron._i18n import _

import six


class AgentMembershipNotFound(nexception.NotFound):
    message = _("Agent membership %(id)s does not exist")


AGENT_MEMBERSHIP = 'agent_membership'
AGENT_MEMBERSHIPS = '%ss' % AGENT_MEMBERSHIP

# Attribute Map
RESOURCE_ATTRIBUTE_MAP = {

    'agent_memberships': {
        'id': {'allow_post': True, 'allow_put': False,
               'validate': {'type:uuid': None},
               'is_visible': True,
               'primary_key': True,
               'required': True},
        'ip_address': {'allow_post': True, 'allow_put': False,
                       'validate': {'type:ip_address': None},
                       'required': True,
                       'is_visible': True},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'required_by_policy': True,
                      'is_visible': False}

    }
}


class Agent_membership(extensions.ExtensionDescriptor):
    """Agent membership extension."""

    @classmethod
    def get_name(cls):
        return "Midonet Agent Membership Extension"

    @classmethod
    def get_alias(cls):
        return "agent-membership"

    @classmethod
    def get_description(cls):
        return "The agent memberships extension."

    @classmethod
    def get_namespace(cls):
        return "http://docs.openstack.org/ext/agent_membership/api/v2.0"

    @classmethod
    def get_updated(cls):
        return "2015-02-26T10:00:00-00:00"

    @classmethod
    def get_resources(cls):
        """Returns Ext Resources."""
        exts = []
        plugin = manager.NeutronManager.get_plugin()

        resource_name = AGENT_MEMBERSHIP
        collection_name = AGENT_MEMBERSHIPS
        params = RESOURCE_ATTRIBUTE_MAP.get(collection_name, dict())
        agent_membership_controller = base.create_resource(
            collection_name, resource_name, plugin, params)
        ex = extensions.ResourceExtension(collection_name,
                                          agent_membership_controller)
        exts.append(ex)

        return exts

    def get_extended_resources(self, version):
        if version == "2.0":
            return RESOURCE_ATTRIBUTE_MAP
        else:
            return {}


@six.add_metaclass(abc.ABCMeta)
class AgentMembershipPluginBase(object):

    @abc.abstractmethod
    def create_agent_membership(self, context, agent_membership):
        pass

    @abc.abstractmethod
    def delete_agent_membership(self, context, id):
        pass

    @abc.abstractmethod
    def get_agent_memberships(self, context, filters=None, fields=None,
                              sorts=None, limit=None, marker=None,
                              page_reverse=False):
        pass

    @abc.abstractmethod
    def get_agent_membership(self, context, id, fields=None):
        pass



# Copyright (C) 2015 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from midonet.neutron._i18n import _LE
from midonet.neutron.client import base as c_base
from midonet.neutron.db import loadbalancer_db as mn_lb_db

from neutron.common import exceptions as n_exc
from neutron.plugins.common import constants
from neutron_lbaas.db.loadbalancer import loadbalancer_db as ldb
from neutron_lbaas.services.loadbalancer.drivers import abstract_driver

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import excutils

LOG = logging.getLogger(__name__)


class MidonetLoadbalancerDriver(abstract_driver.LoadBalancerAbstractDriver,
                                mn_lb_db.LoadBalancerDriverDbMixin):

    def __init__(self, plugin):
        self.plugin = plugin
        self.client = c_base.load_client(cfg.CONF.MIDONET)

    @property
    def core_plugin(self):
        return self.plugin._core_plugin

    def create_vip(self, context, vip):
        LOG.debug("MidonetLoadbalancerDriver.create_vip called: %(vip)r",
                  {'vip': vip})

        try:
            self._validate_vip_subnet(context, vip)
        except n_exc.NeutronException as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to create a vip %(vip_id)s in Midonet: "
                              "%(err)s"), {"vip_id": vip["id"], "err": ex})
                try:
                    self.plugin._delete_db_vip(context, vip['id'])
                except Exception:
                    LOG.exception(_LE("Failed to delete vip %s"), vip['id'])

        self.client.create_vip(context, vip)
        self.plugin.update_status(context, ldb.Vip, vip['id'],
                                  constants.ACTIVE)

        LOG.debug("MidonetLoadbalancerDriver.create_vip exiting: id=%r",
                  vip['id'])

    def delete_vip(self, context, vip):
        LOG.debug("MidonetLoadbalancerDriver.delete_vip called: id=%(vip)r",
                  {'vip': vip})

        self.client.delete_vip(context, vip['id'])
        self.plugin._delete_db_vip(context, vip['id'])

        LOG.debug("MidonetLoadbalancerDriver.delete_vip existing: vip=%(vip)r",
                  {'vip': vip})

    def update_vip(self, context, old_vip, new_vip):
        LOG.debug("MidonetLoadbalancerDriver.update_vip called: "
                  "old_vip=%(old_vip)r, new_vip=%(new_vip)r",
                  {'old_vip': old_vip, 'new_vip': new_vip})

        try:
            self._validate_vip_subnet(context, new_vip)
        except n_exc.NeutronException as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to update a vip %(vip_id)s in Midonet: "
                              "%(err)s"), {"vip_id": old_vip["id"], "err": ex})
                try:
                    self.plugin.update_status(context, ldb.Vip, old_vip["id"],
                                              constants.ERROR)
                except Exception:
                    LOG.exception(_LE("Failed to update vip status %s"),
                                  old_vip['id'])

        self.client.update_vip(context, old_vip['id'], new_vip)
        self.plugin.update_status(context, ldb.Vip, old_vip["id"],
                                  constants.ACTIVE)

        LOG.debug("MidonetLoadbalancerDriver.update_vip exiting: "
                  "old_vip=%(old_vip)r, new_vip=%(new_vip)r",
                  {'old_vip': old_vip, 'new_vip': new_vip})

    def create_pool(self, context, pool):
        LOG.debug("MidonetLoadbalancerDriver.create_pool called: %(pool)r",
                  {'pool': pool})

        try:
            router_id = self._check_and_get_router_id_for_pool(
                context, pool['subnet_id'])
        except n_exc.NeutronException as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to create a pool %(pool_id)s in "
                              "Midonet: %(err)s"),
                          {"pool_id": pool["id"], "err": ex})
                try:
                    self.plugin._delete_db_pool(context, pool['id'])
                except Exception:
                    LOG.exception(_LE("Failed to delete pool %s"), pool['id'])

        pool.update({'router_id': router_id, 'status': constants.ACTIVE})
        self.client.create_pool(context, pool)
        self.plugin.update_status(context, ldb.Pool, pool['id'],
                                  constants.ACTIVE)

        LOG.debug("MidonetLoadbalancerDriver.create_pool exiting: %(pool)r",
                  {'pool': pool})

    def delete_pool(self, context, pool):
        LOG.debug("MidonetLoadbalancerDriver.delete_pool called: %(pool)r",
                  {'pool': pool})

        self.client.delete_pool(context, pool['id'])
        self.plugin._delete_db_pool(context, pool['id'])

        LOG.debug("MidonetLoadbalancerDriver.delete_pool exiting: %(pool)r",
                  {'pool': pool})

    def update_pool(self, context, old_pool, new_pool):
        LOG.debug("MidonetLoadbalancerDriver.update_pool called: "
                  "old_pool=%(old_pool)r, new_pool=%(new_pool)r",
                  {'old_pool': old_pool, 'new_pool': new_pool})

        self.client.update_pool(context, old_pool['id'], new_pool)
        self.plugin.update_status(context, ldb.Pool, old_pool["id"],
                                  constants.ACTIVE)

        LOG.debug("MidonetLoadbalancerDriver.update_pool exiting: "
                  "new_pool=%(new_pool)r", {'new_pool': new_pool})

    def create_member(self, context, member):
        LOG.debug("MidonetLoadbalancerDriver.create_member called: %(member)r",
                  {'member': member})

        self.client.create_member(context, member)
        self.plugin.update_status(context, ldb.Member, member['id'],
                                  constants.ACTIVE)

        LOG.debug("MidonetLoadbalancerDriver.create_member exiting: "
                  "%(member)r", {'member': member})

    def delete_member(self, context, member):
        LOG.debug("MidonetLoadbalancerDriver.delete_member called: %(member)r",
                  {'member': member})

        self.client.delete_member(context, member['id'])
        self.plugin._delete_db_member(context, member['id'])

        LOG.debug("MidonetLoadbalancerDriver.delete_member exiting: "
                  "%(member)r", {'member': member})

    def update_member(self, context, old_member, new_member):
        LOG.debug("MidonetLoadbalancerDriver.update_member called: "
                  "old_member=%(old_member)r, new_member=%(new_member)r",
                  {'old_member': old_member, 'new_member': new_member})

        self.client.update_member(context, old_member['id'], new_member)
        self.plugin.update_status(context, ldb.Member, old_member["id"],
                                  constants.ACTIVE)

        LOG.debug("MidonetLoadbalancerDriver.update_member exiting: "
                  "new_member=%(new_member)r", {'new_member': new_member})

    def create_pool_health_monitor(self, context, health_monitor, pool_id):
        LOG.debug("MidonetLoadbalancerDriver.create_pool_health_monitor "
                  "called: hm=%(health_monitor)r, pool_id=%(pool_id)r",
                  {'health_monitor': health_monitor, 'pool_id': pool_id})

        try:
            self._validate_pool_hm_assoc(context, pool_id,
                                         health_monitor['id'])
        except n_exc.NeutronException as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to create a pool-hm association "
                              "in Midonet: pool=%(pool_id)s, hm=%(hm_id)s, "
                              "%(err)s"),
                          {"pool_id": pool_id, "hm_id": health_monitor['id'],
                           "err": ex})
                try:
                    self.plugin._delete_db_pool_health_monitor(
                        context, health_monitor['id'], pool_id)
                except Exception:
                    LOG.exception(_LE("Failed to delete pool-hm association:"
                                      "pool_id=%(pool_id)s, hm_id=%(hm_id)s"),
                                  {"pool_id": pool_id,
                                   "hm_id": health_monitor['id']})

        self.client.create_health_monitor(context, health_monitor)
        self.plugin.update_pool_health_monitor(context, health_monitor['id'],
                                               pool_id, constants.ACTIVE, "")

        LOG.debug("MidonetLoadbalancerDriver.create_pool_health_monitor "
                  "exiting: %(health_monitor)r, %(pool_id)r",
                  {'health_monitor': health_monitor, 'pool_id': pool_id})

    def delete_pool_health_monitor(self, context, health_monitor, pool_id):
        LOG.debug("MidonetLoadbalancerDriver.delete_pool_health_monitor "
                  "called: health_monitor=%(health_monitor)r, "
                  "pool_id=%(pool_id)r", {'health_monitor': health_monitor,
                                          'pool_id': pool_id})

        self.client.delete_health_monitor(context, health_monitor['id'])
        self.plugin._delete_db_pool_health_monitor(context,
                                                   health_monitor['id'],
                                                   pool_id)

        LOG.debug("MidonetLoadbalancerDriver.delete_pool_health_monitor "
                  "exiting: %(health_monitor)r, %(pool_id)r",
                  {'health_monitor': health_monitor, 'pool_id': pool_id})

    def update_pool_health_monitor(self, context, old_health_monitor,
                                   health_monitor, pool_id):
        LOG.debug("MidonetLoadbalancerDriver.update_pool_health_monitor "
                  "called: old_health_monitor=%(old_health_monitor)r, "
                  "health_monitor=%(health_monitor)r, pool_id=%(pool_id)r",
                  {'old_health_monitor': old_health_monitor,
                   'health_monitor': health_monitor, 'pool_id': pool_id})

        self.client.update_health_monitor(context, old_health_monitor['id'],
                                          health_monitor)

        LOG.debug("MidonetLoadbalancerDriver.update_pool_health_monitor "
                  "exiting: health_monitor=%(health_monitor)r",
                  {'health_monitor': health_monitor})

    def stats(self, context, pool_id):
        raise NotImplementedError()



# Copyright (C) 2015 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron.common import constants as n_const
from neutron.common import exceptions as n_exc
from neutron.db import l3_db
from neutron.db import models_v2
from neutron import manager
from neutron.plugins.common import constants as service_constants

from midonet.neutron._i18n import _LE

from sqlalchemy.orm import exc


class LoadBalancerDriverDbMixin(object):

    def _check_and_get_router_id_for_pool(self, context, subnet_id):

        subnet = self.core_plugin._get_subnet(context, subnet_id)

        # Check whether the network is external
        if self._is_subnet_external(context, subnet):
            msg = (_LE("pool subnet must not be public"))
            raise n_exc.BadRequest(resource='pool', msg=msg)

        router_id = self._get_router_from_subnet(context, subnet)
        if not router_id:
            msg = (_LE("pool subnet must be associated with router"))
            raise n_exc.BadRequest(resource='pool', msg=msg)
        return router_id

    def _get_router_from_pool(self, context, pool):
        subnet = self.core_plugin._get_subnet(context, pool['subnet_id'])
        return self._get_router_from_subnet(context, subnet)

    def _get_router_from_port(self, context, port_id):
        routers = context.session.query(l3_db.Router)
        routers = routers.join(models_v2.Port,
                               l3_db.Router.id == models_v2.Port.device_id)
        routers = routers.filter(models_v2.Port.id == port_id)
        try:
            return routers.one().id
        except exc.NoResultFound:
            return None

    def _get_router_from_subnet(self, context, subnet):
        iport = self._get_router_interface_port(context, subnet)
        if iport is None:
            return None
        else:
            return self._get_router_from_port(context, iport)

    def _get_router_interface_port(self, context, subnet):
        all_ports = context.session.query(models_v2.Port).join(
            models_v2.Port.fixed_ips)
        ports = all_ports.filter(
            models_v2.Port.device_owner == n_const.DEVICE_OWNER_ROUTER_INTF)
        ports = ports.filter(models_v2.Port.network_id == subnet['network_id'])
        ports = ports.filter(
            models_v2.IPAllocation.ip_address == subnet['gateway_ip'])
        try:
            return ports.one().id
        except exc.NoResultFound:
            return None

    def _is_subnet_external(self, context, subnet):
        network = self.core_plugin._get_network(context, subnet['network_id'])
        return network.external

    def _validate_vip_subnet(self, context, vip):
        subnet = self.core_plugin._get_subnet(context, vip['subnet_id'])
        pool = self.plugin.get_pool(context, vip['pool_id'])

        # VIP and pool must not be on the same subnet if pool is associated
        # with a health monitor.  Health monitor would not work in that case.
        if pool['health_monitors'] and pool['subnet_id'] == subnet['id']:
            msg = (_LE("The VIP and pool cannot be on the same subnet if "
                       "health monitor is associated"))
            raise n_exc.BadRequest(resource='vip', msg=msg)

        if not self._is_subnet_external(context, subnet):
            return

        # ensure that if the vip subnet is public, the router has its
        # gateway set.
        router_id = self._get_router_from_pool(context, pool)

        # router_id should never be None because it was already validated
        # when we created the pool
        assert router_id is not None

        l3plugin = manager.NeutronManager.get_service_plugins().get(
            service_constants.L3_ROUTER_NAT)
        router = l3plugin._get_router(context, router_id)
        if router.get('gw_port_id') is None:
            msg = (_LE("The router must have its gateway set if the "
                       "VIP subnet is external"))
            raise n_exc.BadRequest(resource='vip', msg=msg)

    def _validate_pool_hm_assoc(self, context, pool_id, hm_id):
        pool = self.plugin.get_pool(context, pool_id)
        assoc = next((x for x in pool['health_monitors'] if x != hm_id), None)

        # There is an association with a different health monitor
        if assoc:
            msg = (_LE("The pool is already associated with a different "
                       "health monitor"))
            raise n_exc.BadRequest(resource='pool_monitor_association',
                                   msg=msg)

        # When associating health monitor, the subnet of VIP and Pool must not
        # match
        if pool['vip_id']:
            vip = self.plugin.get_vip(context, pool['vip_id'])
            if vip['subnet_id'] == pool['subnet_id']:
                msg = (_LE("The VIP and pool cannot be on the same subnet if "
                           "health monitor is associated"))
                raise n_exc.BadRequest(resource='pool_monitor_association',
                                       msg=msg)



# Copyright (C) 2012 Midokura Japan K.K.
# Copyright (C) 2013 Midokura PTE LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_db import exception as oslo_db_exc
from oslo_log import log as logging
from oslo_utils import excutils

from neutron.common import constants as n_const
from neutron.common import exceptions as n_exc
from neutron.db import api as db_api
from neutron.db import l3_db
from neutron.db import l3_gwmode_db
from neutron.extensions import extra_dhcp_opt as edo_ext
from neutron.extensions import securitygroup as ext_sg

from midonet.neutron._i18n import _, _LE, _LW
from midonet.neutron.common import constants
from midonet.neutron.common import utils as c_utils
from midonet.neutron import plugin


LOG = logging.getLogger(__name__)


midonet_opts = [
    cfg.StrOpt('midonet_uri', default='http://localhost:8080/midonet-api',
               help=_('MidoNet API server URI.')),
    cfg.StrOpt('username', default='admin',
               help=_('MidoNet admin username.')),
    cfg.StrOpt('password', default='passw0rd',
               secret=True,
               help=_('MidoNet admin password.')),
    cfg.StrOpt('project_id',
               default='77777777-7777-7777-7777-777777777777',
               help=_('ID of the project that MidoNet admin user '
                      'belongs to.'))
]


cfg.CONF.register_opts(midonet_opts, "MIDONET")


# TODO(yamamoto): Merge MidonetMixin into MidonetPluginV2
class MidonetMixin(plugin.MidonetMixinBase,
                   l3_gwmode_db.L3_NAT_db_mixin):

    supported_extension_aliases = ['ext-gw-mode', 'extra_dhcp_opt']

    def create_network(self, context, network):
        LOG.debug('MidonetMixin.create_network called: network=%r', network)

        net_data = network['network']
        tenant_id = net_data['tenant_id']
        net_data['tenant_id'] = tenant_id
        self._ensure_default_security_group(context, tenant_id)

        with context.session.begin(subtransactions=True):
            net = super(MidonetMixin, self).create_network(context, network)
            net_data['id'] = net['id']
            self._process_l3_create(context, net, net_data)
            self.client.create_network_precommit(context, net)

        try:
            self.client.create_network_postcommit(net)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to create a network %(net_id)s "
                              "in Midonet: %(err)s"),
                          {"net_id": net["id"], "err": ex})
                try:
                    self.delete_network(context, net['id'])
                except Exception:
                    LOG.exception(_LE("Failed to delete network %s"),
                                  net['id'])

        # NOTE(kevinbenton): this extra lookup is necessary to get the
        # latest db model for the extension functions
        net_model = self._get_network(context, net['id'])
        self._apply_dict_extend_functions('networks', net, net_model)

        LOG.debug("MidonetMixin.create_network exiting: net=%r", net)
        return net

    def update_network(self, context, id, network):
        LOG.debug("MidonetMixin.update_network called: id=%(id)r, "
                  "network=%(network)r", {'id': id, 'network': network})

        with context.session.begin(subtransactions=True):
            net = super(MidonetMixin, self).update_network(
                context, id, network)
            self._process_l3_update(context, net, network['network'])
            self.client.update_network_precommit(context, id, net)

        self.client.update_network_postcommit(id, net)

        LOG.debug("MidonetMixin.update_network exiting: net=%r", net)
        return net

    def delete_network(self, context, id):
        LOG.debug("MidonetMixin.delete_network called: id=%r", id)

        with context.session.begin(subtransactions=True):
            self._process_l3_delete(context, id)
            try:
                super(MidonetMixin, self).delete_network(context, id)
            except n_exc.NetworkInUse as ex:
                LOG.warning(_LW("Error deleting network %(net)s, retrying..."),
                            {'net': id})
                # Contention for DHCP port deletion and network deletion occur
                # often which leads to NetworkInUse error.  Retry to get
                # around this problem.
                raise oslo_db_exc.RetryRequest(ex)

            self.client.delete_network_precommit(context, id)

        self.client.delete_network_postcommit(id)

        LOG.debug("MidonetMixin.delete_network exiting: id=%r", id)

    def create_subnet(self, context, subnet):
        LOG.debug("MidonetMixin.create_subnet called: subnet=%r", subnet)

        with context.session.begin(subtransactions=True):
            s = super(MidonetMixin, self).create_subnet(context, subnet)
            self.client.create_subnet_precommit(context, s)

        try:
            self.client.create_subnet_postcommit(s)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to create a subnet %(s_id)s in Midonet:"
                              "%(err)s"), {"s_id": s["id"], "err": ex})
                try:
                    self.delete_subnet(context, s['id'])
                except Exception:
                    LOG.exception(_LE("Failed to delete subnet %s"), s['id'])

        LOG.debug("MidonetMixin.create_subnet exiting: subnet=%r", s)
        return s

    def delete_subnet(self, context, id):
        LOG.debug("MidonetMixin.delete_subnet called: id=%s", id)

        with context.session.begin(subtransactions=True):
            super(MidonetMixin, self).delete_subnet(context, id)
            self.client.delete_subnet_precommit(context, id)

        self.client.delete_subnet_postcommit(id)

        LOG.debug("MidonetMixin.delete_subnet exiting")

    def update_subnet(self, context, id, subnet):
        LOG.debug("MidonetMixin.update_subnet called: id=%s", id)

        with context.session.begin(subtransactions=True):
            s = super(MidonetMixin, self).update_subnet(context, id, subnet)
            self.client.update_subnet_precommit(context, id, s)

        self.client.update_subnet_postcommit(id, s)

        LOG.debug("MidonetMixin.update_subnet exiting: subnet=%r", s)
        return s

    def create_port(self, context, port):
        LOG.debug("MidonetMixin.create_port called: port=%r", port)

        port_data = port['port']
        # REVISIT(yamamoto): this nested transaction is a workaround
        # for bug #1490917.
        with db_api.exc_to_retry(oslo_db_exc.DBDuplicateEntry),\
                db_api.autonested_transaction(context.session):
            # Set status along admin_state_up if the parameter is specified.
            if port['port'].get('admin_state_up') is not None:
                if not port['port']['admin_state_up']:
                    port['port']['status'] = n_const.PORT_STATUS_DOWN
            # Create a Neutron port
            new_port = super(MidonetMixin, self).create_port(context, port)

            # Do not create a gateway port if it has no IP address assigned as
            # MidoNet does not yet handle this case.
            if (new_port.get('device_owner') == n_const.DEVICE_OWNER_ROUTER_GW
                    and not new_port['fixed_ips']):
                msg = (_("No IPs assigned to the gateway port for"
                         " router %s") % port_data['device_id'])
                raise n_exc.BadRequest(resource='router', msg=msg)

            dhcp_opts = port['port'].get(edo_ext.EXTRADHCPOPTS, [])

            # Make sure that the port created is valid
            if "id" not in new_port:
                raise n_exc.BadRequest(resource='port',
                                       msg="Invalid port created")

            # Update fields
            port_data.update(new_port)

            # Bind security groups to the port
            self._ensure_default_security_group_on_port(context, port)
            sg_ids = self._get_security_groups_on_port(context, port)
            self._process_port_create_security_group(context, new_port, sg_ids)

            # Process port bindings
            self._process_portbindings_create_and_update(context, port_data,
                                                         new_port)

            self._process_port_create_extra_dhcp_opts(context, new_port,
                                                      dhcp_opts)
            self.client.create_port_precommit(context, new_port)

        try:
            self.client.create_port_postcommit(new_port)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to create a port %(new_port)s: %(err)s"),
                          {"new_port": new_port, "err": ex})
                try:
                    self.delete_port(context, new_port['id'],
                                     l3_port_check=False)
                except Exception:
                    LOG.exception(_LE("Failed to delete port %s"),
                                  new_port['id'])

        # NOTE(kevinbenton): this extra lookup is necessary to get the
        # latest db model for the extension functions
        port_model = self._get_port(context, new_port['id'])
        self._apply_dict_extend_functions('ports', new_port, port_model)

        LOG.debug("MidonetMixin.create_port exiting: port=%r", new_port)
        return new_port

    def delete_port(self, context, id, l3_port_check=True):
        LOG.debug("MidonetMixin.delete_port called: id=%(id)s "
                  "l3_port_check=%(l3_port_check)r",
                  {'id': id, 'l3_port_check': l3_port_check})

        # if needed, check to see if this is a port owned by
        # and l3-router.  If so, we should prevent deletion.
        if l3_port_check:
            self.prevent_l3_port_deletion(context, id)

        with context.session.begin(subtransactions=True):
            super(MidonetMixin, self).disassociate_floatingips(
                context, id, do_notify=False)
            super(MidonetMixin, self).delete_port(context, id)
            self.client.delete_port_precommit(context, id)

        self.client.delete_port_postcommit(id)

        LOG.debug("MidonetMixin.delete_port exiting: id=%r", id)

    def update_port(self, context, id, port):
        LOG.debug("MidonetMixin.update_port called: id=%(id)s port=%(port)r",
                  {'id': id, 'port': port})

        with db_api.exc_to_retry(oslo_db_exc.DBDuplicateEntry),\
                context.session.begin(subtransactions=True):

            # update the port DB
            original_port = self.get_port(context, id)
            p = super(MidonetMixin, self).update_port(context, id, port)
            c_utils.check_update_port(original_port, p)

            has_sg = self._check_update_has_security_groups(port)
            delete_sg = self._check_update_deletes_security_groups(port)

            if delete_sg or has_sg:
                # delete the port binding and read it with the new rules.
                self._delete_port_security_group_bindings(context, id)
                sg_ids = self._get_security_groups_on_port(context, port)
                self._process_port_create_security_group(context, p, sg_ids)
            self._update_extra_dhcp_opts_on_port(context, id, port, p)

            self._process_portbindings_create_and_update(context,
                                                         port['port'], p)

            self.client.update_port_precommit(context, id, p)

        # Set status along admin_state_up to update.
        if p['admin_state_up']:
            update_status = n_const.PORT_STATUS_ACTIVE
        else:
            update_status = n_const.PORT_STATUS_DOWN

        try:
            self.client.update_port_postcommit(id, p)
            data = {'port': {'status': update_status}}
            super(MidonetMixin, self).update_port(context, id, data)
            # prevent binding:profile information from lacking
            p['status'] = data['port']['status']
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to update a port %(port_id)s in "
                              "MidoNet: %(err)s"), {"port_id": id, "err": ex})
                try:
                    data = {'port': {'status': n_const.PORT_STATUS_ERROR}}
                    super(MidonetMixin, self).update_port(context, id, data)
                except Exception:
                    LOG.exception(_LE("Failed to update port status %s"), id)

        LOG.debug("MidonetMixin.update_port exiting: p=%r", p)
        return p

    def create_router(self, context, router):
        LOG.debug("MidonetMixin.create_router called: router=%(router)s",
                  {"router": router})

        with context.session.begin(subtransactions=True):
            r = super(MidonetMixin, self).create_router(context, router)
            self.client.create_router_precommit(context, r)

        try:
            self.client.create_router_postcommit(r)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to create a router %(r_id)s in Midonet:"
                              "%(err)s"), {"r_id": r["id"], "err": ex})
                try:
                    self.delete_router(context, r['id'])
                except Exception:
                    LOG.exception(_LE("Failed to delete a router %s"), r["id"])

        LOG.debug("MidonetMixin.create_router exiting: router=%(router)s.",
                  {"router": r})
        return r

    def update_router(self, context, id, router):
        LOG.debug("MidonetMixin.update_router called: id=%(id)s "
                  "router=%(router)r", {"id": id, "router": router})

        with context.session.begin(subtransactions=True):
            r = super(MidonetMixin, self).update_router(context, id, router)
            self.client.update_router_precommit(context, id, r)

        try:
            self.client.update_router_postcommit(id, r)
            if r['status'] != constants.ROUTER_STATUS_ACTIVE:
                data = {'router': {'status': constants.ROUTER_STATUS_ACTIVE}}
                r = super(MidonetMixin,
                        self).update_router(context, id, data)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to update a router %(r_id)s in MidoNet: "
                              "%(err)s"), {"r_id": id, "err": ex})
                try:
                    data = {'router':
                        {'status': constants.ROUTER_STATUS_ERROR}}
                    super(MidonetMixin, self).update_router(context, id, data)
                except Exception:
                    LOG.exception(_LE("Failed to update a router "
                                      "status %s"), id)

        LOG.debug("MidonetMixin.update_router exiting: router=%r", r)
        return r

    def delete_router(self, context, id):
        LOG.debug("MidonetMixin.delete_router called: id=%s", id)

        with context.session.begin(subtransactions=True):
            super(MidonetMixin, self).delete_router(context, id)
            self.client.delete_router_precommit(context, id)

        self.client.delete_router_postcommit(id)

        LOG.debug("MidonetMixin.delete_router exiting: id=%s", id)

    def add_router_interface(self, context, router_id, interface_info):
        LOG.debug("MidonetMixin.add_router_interface called: "
                  "router_id=%(router_id)s, interface_info=%(interface_info)r",
                  {'router_id': router_id, 'interface_info': interface_info})

        by_port = bool(interface_info.get('port_id'))
        with context.session.begin(subtransactions=True):
            info = super(MidonetMixin, self).add_router_interface(
                context, router_id, interface_info)
            self.client.add_router_interface_precommit(context, router_id,
                                                       info)

        try:
            self.client.add_router_interface_postcommit(router_id, info)
        except Exception as ex:
            LOG.error(_LE("Failed to create MidoNet resources to add router "
                          "interface. info=%(info)s, router_id=%(router_id)s, "
                          "error=%(err)r"),
                      {"info": info, "router_id": router_id, "err": ex})
            with excutils.save_and_reraise_exception():
                if not by_port:
                    self.remove_router_interface(context, router_id, info)

        LOG.debug("MidonetMixin.add_router_interface exiting: info=%r", info)
        return info

    def remove_router_interface(self, context, router_id, interface_info):
        LOG.debug("MidonetMixin.remove_router_interface called: "
                  "router_id=%(router_id)s, interface_info=%(interface_info)r",
                  {'router_id': router_id, 'interface_info': interface_info})

        with context.session.begin(subtransactions=True):
            info = super(MidonetMixin, self).remove_router_interface(
                context, router_id, interface_info)
            self.client.remove_router_interface_precommit(context, router_id,
                                                          info)

        self.client.remove_router_interface_postcommit(router_id, info)

        LOG.debug("MidonetMixin.remove_router_interface exiting: info=%r",
                  info)
        return info

    def create_floatingip(self, context, floatingip):
        LOG.debug("MidonetMixin.create_floatingip called: ip=%r", floatingip)

        with context.session.begin(subtransactions=True):
            fip = super(MidonetMixin, self).create_floatingip(context,
                                                              floatingip)
            self.client.create_floatingip_precommit(context, fip)

        try:
            self.client.create_floatingip_postcommit(fip)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to create floating ip %(fip)s: %(err)s"),
                          {"fip": fip, "err": ex})
                try:
                    self.delete_floatingip(context, fip['id'])
                except Exception:
                    LOG.exception(_LE("Failed to delete a floating ip %s"),
                                  fip['id'])

        LOG.debug("MidonetMixin.create_floatingip exiting: fip=%r", fip)
        return fip

    def delete_floatingip(self, context, id):
        LOG.debug("MidonetMixin.delete_floatingip called: id=%s", id)

        with context.session.begin(subtransactions=True):
            super(MidonetMixin, self).delete_floatingip(context, id)
            self.client.delete_floatingip_precommit(context, id)

        self.client.delete_floatingip_postcommit(id)

        LOG.debug("MidonetMixin.delete_floatingip exiting: id=%r", id)

    def update_floatingip(self, context, id, floatingip):
        LOG.debug("MidonetMixin.update_floatingip called: id=%(id)s "
                  "floatingip=%(floatingip)s ",
                  {'id': id, 'floatingip': floatingip})

        with context.session.begin(subtransactions=True):
            fip = super(MidonetMixin, self).update_floatingip(context, id,
                                                              floatingip)
            self.client.update_floatingip_precommit(context, id, fip)

            # Update status based on association
            if fip.get('port_id') is None:
                fip['status'] = n_const.FLOATINGIP_STATUS_DOWN
            else:
                fip['status'] = n_const.FLOATINGIP_STATUS_ACTIVE
            self.update_floatingip_status(context, id, fip['status'])

        try:
            self.client.update_floatingip_postcommit(id, fip)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to update a floating ip "
                              "%(fip_id)s in MidoNet: "
                              "%(err)s"), {"fip_id": id, "err": ex})
                try:
                    self.update_floatingip_status(
                            context, id, n_const.FLOATINGIP_STATUS_ERROR)
                except Exception:
                    LOG.exception(_LE("Failed to update floating ip "
                                      "status %s"), id)

        LOG.debug("MidonetMixin.update_floating_ip exiting: fip=%s", fip)
        return fip

    def create_security_group(self, context, security_group, default_sg=False):
        LOG.debug("MidonetMixin.create_security_group called: "
                  "security_group=%(security_group)s "
                  "default_sg=%(default_sg)s ",
                  {'security_group': security_group, 'default_sg': default_sg})

        sg = security_group.get('security_group')
        tenant_id = sg['tenant_id']
        if not default_sg:
            self._ensure_default_security_group(context, tenant_id)

        # Create the Neutron sg first
        with context.session.begin(subtransactions=True):
            sg = super(MidonetMixin, self).create_security_group(
                context, security_group, default_sg)
            self.client.create_security_group_precommit(context, sg)

        try:
            self.client.create_security_group_postcommit(sg)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE("Failed to create MidoNet resources for "
                              "sg %(sg)r, error=%(err)r"),
                          {"sg": sg, "err": ex})
                try:
                    self.delete_security_group(context, sg['id'])
                except Exception:
                    LOG.exception(_LE("Failed to delete a security group %s"),
                                  sg['id'])

        LOG.debug("MidonetMixin.create_security_group exiting: sg=%r", sg)
        return sg

    def delete_security_group(self, context, id):
        LOG.debug("MidonetMixin.delete_security_group called: id=%s", id)

        sg = super(MidonetMixin, self).get_security_group(context, id)
        if not sg:
            raise ext_sg.SecurityGroupNotFound(id=id)

        if sg["name"] == 'default' and not context.is_admin:
            raise ext_sg.SecurityGroupCannotRemoveDefault()

        with context.session.begin(subtransactions=True):
            super(MidonetMixin, self).delete_security_group(context, id)
            self.client.delete_security_group_precommit(context, id)

        self.client.delete_security_group_postcommit(id)

        LOG.debug("MidonetMixin.delete_security_group exiting: id=%r", id)

    def create_security_group_rule(self, context, security_group_rule):
        LOG.debug("MidonetMixin.create_security_group_rule called: "
                  "security_group_rule=%(security_group_rule)r",
                  {'security_group_rule': security_group_rule})

        with context.session.begin(subtransactions=True):
            rule = super(MidonetMixin, self).create_security_group_rule(
                context, security_group_rule)
            self.client.create_security_group_rule_precommit(context, rule)

        try:
            self.client.create_security_group_rule_postcommit(rule)
        except Exception as ex:
            with excutils.save_and_reraise_exception():
                LOG.error(_LE('Failed to create security group rule %(sg)s,'
                          'error: %(err)s'), {'sg': rule, 'err': ex})
                try:
                    self.delete_security_group_rule(context, rule['id'])
                except Exception:
                    LOG.exception(_LE("Failed to delete "
                                      "a security group rule %s"), rule['id'])

        LOG.debug("MidonetMixin.create_security_group_rule exiting: rule=%r",
                  rule)
        return rule

    def create_security_group_rule_bulk(self, context, security_group_rules):
        LOG.debug("MidonetMixin.create_security_group_rule_bulk called: "
                  "security_group_rules=%(security_group_rules)r",
                  {'security_group_rules': security_group_rules})

        with context.session.begin(subtransactions=True):
            rules = super(
                MidonetMixin, self).create_security_group_rule_bulk_native(
                    context, security_group_rules)
            self.client.create_security_group_rule_bulk_precommit(context,
                                                                  rules)

        try:
            self.client.create_security_group_rule_bulk_postcommit(rules)
        except Exception as ex:
            LOG.error(_LE("Failed to create bulk security group rules %(sg)s, "
                          "error: %(err)s"), {"sg": rules, "err": ex})
            with excutils.save_and_reraise_exception():
                for rule in rules:
                    self.delete_security_group_rule(context, rule['id'])

        LOG.debug("MidonetMixin.create_security_group_rule_bulk exiting: "
                  "rules=%r", rules)
        return rules

    def delete_security_group_rule(self, context, sg_rule_id):
        LOG.debug("MidonetMixin.delete_security_group_rule called: "
                  "sg_rule_id=%s", sg_rule_id)

        with context.session.begin(subtransactions=True):
            super(MidonetMixin, self).delete_security_group_rule(context,
                                                                 sg_rule_id)
            self.client.delete_security_group_rule_precommit(context,
                                                             sg_rule_id)

        self.client.delete_security_group_rule_postcommit(sg_rule_id)

        LOG.debug("MidonetMixin.delete_security_group_rule exiting: id=%r",
                  id)


class MidonetPluginV2(MidonetMixin):

    vendor_extensions = MidonetMixin.supported_extension_aliases
    # NOTE(yamamoto): While the order in this list doesn't matter
    # functionality-wise, it's alphabetically sorted for easier comparison.
    supported_extension_aliases = [
        'agent',
        'binding',
        'dhcp_agent_scheduler',
        'external-net',
        'quotas',
        'router',
        'security-group',
        'subnet_allocation',
    ] + vendor_extensions

    __native_bulk_support = True

    def __init__(self):
        super(MidonetPluginV2, self).__init__()
        l3_db.subscribe()


def list_opts():
    return [
        ('midonet', midonet_opts),
    ]



# Copyright (C) 2016 Midokura SARL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from midonet.neutron._i18n import _
from midonet.neutron.common import constants
from neutron.api import extensions
from neutron.api.v2 import attributes as attr
from neutron.api.v2 import base
from neutron.api.v2 import resource_helper
from neutron import manager
from neutron.quota import resource_registry
from neutron_lib import exceptions as nexception
from oslo_config import cfg
import six


class LoggingResourceNotFound(nexception.NotFound):
    message = _("Logging resource %(id)s does not exist")


class FirewallLogNotFound(nexception.NotFound):
    message = _("Firewall log %(id)s does not exist")


class ResourceInUseByLoggingResource(nexception.InUse):
    message = _("%(resource_name)s %(resource_id)s %(reason)s")

    def __init__(self, **kwargs):
        if 'reason' not in kwargs:
            kwargs['reason'] = "is in use by logging resource"
        super(ResourceInUseByLoggingResource, self).__init__(**kwargs)


LOGGING_PREFIX = '/logging'
FW_EVENT_ACCEPT = 'ACCEPT'
FW_EVENT_DROP = 'DROP'
FW_EVENT_ALL = 'ALL'
FW_EVENTS = [FW_EVENT_ACCEPT, FW_EVENT_DROP, FW_EVENT_ALL]
LOG_COMMON_FIELDS = {
    'id': {'allow_post': False, 'allow_put': False,
           'validate': {'type:uuid': None},
           'is_visible': True, 'primary_key': True},
    'tenant_id': {'allow_post': True, 'allow_put': False,
                  'required_by_policy': True, 'is_visible': True},
    'logging_resource_id': {'allow_post': False, 'allow_put': False,
                            'is_visible': True}
}

RESOURCE_ATTRIBUTE_MAP = {
    'logging_resources': {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None}, 'is_visible': True,
               'primary_key': True},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'required_by_policy': True, 'is_visible': True},
        'name': {'allow_post': True, 'allow_put': True,
                 'validate': {'type:string': attr.NAME_MAX_LEN},
                 'default': '', 'is_visible': True},
        'description': {
            'allow_post': True, 'allow_put': True,
            'validate': {'type:string': attr.LONG_DESCRIPTION_MAX_LEN},
            'default': '', 'is_visible': True},
        'enabled': {'allow_post': True, 'allow_put': True,
                    'is_visible': True, 'default': False,
                    'convert_to': attr.convert_to_boolean},
        'firewall_logs': {'allow_post': False, 'allow_put': False,
                          'is_visible': True}
    }
}

SUB_RESOURCE_ATTRIBUTE_MAP = {
    'firewall_logs': {
        'parent': {'collection_name': 'logging_resources',
                   'member_name': 'logging_resource'},
        'parameters': dict((LOG_COMMON_FIELDS),
                      **{
                        'description': {
                            'allow_post': True, 'allow_put': True,
                            'validate': {
                                'type:string':
                                    attr.LONG_DESCRIPTION_MAX_LEN},
                            'default': '', 'is_visible': True},
                        'firewall_id': {
                            'allow_post': True, 'allow_put': False,
                            'is_visible': True,
                            'validate': {'type:uuid': None}},
                        'fw_event': {
                            'allow_post': True, 'allow_put': True,
                            'is_visible': True,
                            'validate': {'type:values': FW_EVENTS},
                            'default': FW_EVENT_ALL}
                      })
    },
}

# A tenant may want to create firewall logs for all firewalls in tenant.
# Set default quotas to align with default quota_firewall of 10.

firewall_log_quota_opts = [
    cfg.IntOpt('quota_firewall_log',
               default=10,
               help=_('Number of firewall logs allowed per tenant. '
                      'A negative value means unlimited.'))
]
cfg.CONF.register_opts(firewall_log_quota_opts, 'QUOTAS')


class Logging_resource(extensions.ExtensionDescriptor):
    """Logging resource extension."""

    @classmethod
    def get_name(cls):
        return "Midonet Logging Resource Extension"

    @classmethod
    def get_alias(cls):
        return "logging-resource"

    @classmethod
    def get_description(cls):
        return "The logging resource extension."

    @classmethod
    def get_namespace(cls):
        return "http://docs.openstack.org/ext/logging_resource/api/v2.0"

    @classmethod
    def get_updated(cls):
        return "2016-06-06T10:00:00-00:00"

    @classmethod
    def get_resources(cls):
        """Returns Ext Resources."""

        plural_mappings = resource_helper.build_plural_mappings(
            {}, RESOURCE_ATTRIBUTE_MAP)

        attr.PLURALS.update(plural_mappings)
        resources = resource_helper.build_resource_info(
            plural_mappings,
            RESOURCE_ATTRIBUTE_MAP,
            constants.LOGGING_RESOURCE)
        plugin = manager.NeutronManager.get_service_plugins()[
            constants.LOGGING_RESOURCE]

        for collection_name in SUB_RESOURCE_ATTRIBUTE_MAP:
            resource_name = collection_name[:-1]
            resource_registry.register_resource_by_name(resource_name)
            parent = SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].get('parent')
            params = SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].get(
                'parameters')

            controller = base.create_resource(collection_name, resource_name,
                                              plugin, params,
                                              allow_bulk=True,
                                              parent=parent)

            resource = extensions.ResourceExtension(
                collection_name,
                controller, parent,
                path_prefix=LOGGING_PREFIX,
                attr_map=params)
            resources.append(resource)

        return resources

    def get_extended_resources(self, version):
        if version == "2.0":
            return RESOURCE_ATTRIBUTE_MAP
        else:
            return {}


@six.add_metaclass(abc.ABCMeta)
class LoggingResourcePluginBase(object):

    path_prefix = LOGGING_PREFIX

    @abc.abstractmethod
    def create_logging_resource(self, context, logging_resource):
        pass

    @abc.abstractmethod
    def update_logging_resource(self, context, id, logging_resource):
        pass

    @abc.abstractmethod
    def delete_logging_resource(self, context, id):
        pass

    @abc.abstractmethod
    def get_logging_resources(self, context, filters=None, fields=None,
                              sorts=None, limit=None, marker=None,
                              page_reverse=False):
        pass

    @abc.abstractmethod
    def get_logging_resource(self, context, id, fields=None):
        pass

    @abc.abstractmethod
    def create_logging_resource_firewall_log(self, context,
                                             logging_resource_id,
                                             firewall_log):
        pass

    @abc.abstractmethod
    def update_logging_resource_firewall_log(
            self, context, id, logging_resource_id, firewall_log):
        pass

    @abc.abstractmethod
    def delete_logging_resource_firewall_log(self, context,
                                             id, logging_resource_id):
        pass

    @abc.abstractmethod
    def get_logging_resource_firewall_logs(self, context, logging_resource_id,
                                           filters=None, fields=None,
                                           sorts=None, limit=None,
                                           marker=None, page_reverse=False):
        pass

    @abc.abstractmethod
    def get_logging_resource_firewall_log(self, context, id,
                                          logging_resource_id, fields=None):
        pass



# Copyright (C) 2015 Midokura SARL.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import excutils

from neutron.callbacks import events
from neutron.callbacks import exceptions
from neutron.callbacks import registry
from neutron.callbacks import resources
from neutron.db import l3_gwmode_db
from neutron.extensions import l3


class MidonetL3DBMixin(l3_gwmode_db.L3_NAT_db_mixin):
    # TODO(kengo): This is temporary workaround until upstream adds a check
    # for router deletion in l3_db

    def _check_router_not_in_use(self, context, router_id):
        try:
            kwargs = {'context': context, 'router_id': router_id}
            registry.notify(
                resources.ROUTER, events.BEFORE_DELETE, self, **kwargs)
        except exceptions.CallbackFailure as e:
            with excutils.save_and_reraise_exception():
                if len(e.errors) == 1:
                    raise e.errors[0].error
                raise l3.RouterInUse(router_id=router_id, reason=e)
